@@ -1,3 +1,4 @@
+# frozen_string_literal: true
 require "spec_helper"
 
 describe "bundle platform" do
@@ -12,7 +13,7 @@ describe "bundle platform" do
       G
 
       bundle "platform"
-      out.should == <<-G.chomp
+      expect(out).to eq(<<-G.chomp)
 Your platform is: #{RUBY_PLATFORM}
 
 Your app has gems that work on these platforms:
@@ -21,6 +22,29 @@ Your app has gems that work on these platforms:
 Your Gemfile specifies a Ruby version requirement:
 * ruby #{RUBY_VERSION}
 
+Your current platform satisfies the Ruby version requirement.
+G
+    end
+
+    it "returns all the output including the patchlevel" do
+      gemfile <<-G
+        source "file://#{gem_repo1}"
+
+        #{ruby_version_correct_patchlevel}
+
+        gem "foo"
+      G
+
+      bundle "platform"
+      expect(out).to eq(<<-G.chomp)
+Your platform is: #{RUBY_PLATFORM}
+
+Your app has gems that work on these platforms:
+* ruby
+
+Your Gemfile specifies a Ruby version requirement:
+* ruby #{RUBY_VERSION}p#{RUBY_PATCHLEVEL}
+
 Your current platform satisfies the Ruby version requirement.
 G
     end
@@ -33,7 +57,7 @@ G
       G
 
       bundle "platform"
-      out.should == <<-G.chomp
+      expect(out).to eq(<<-G.chomp)
 Your platform is: #{RUBY_PLATFORM}
 
 Your app has gems that work on these platforms:
@@ -53,7 +77,7 @@ G
       G
 
       bundle "platform"
-      out.should == <<-G.chomp
+      expect(out).to eq(<<-G.chomp)
 Your platform is: #{RUBY_PLATFORM}
 
 Your app has gems that work on these platforms:
@@ -78,10 +102,10 @@ G
 
       bundle "platform --ruby"
 
-      out.should eq("ruby 1.9.3")
+      expect(out).to eq("ruby 1.9.3")
     end
 
-    it "engine defaults to MRI" do
+    it "defaults to MRI" do
       gemfile <<-G
         source "file://#{gem_repo1}"
         ruby "1.9.3"
@@ -91,7 +115,7 @@ G
 
       bundle "platform --ruby"
 
-      out.should eq("ruby 1.9.3")
+      expect(out).to eq("ruby 1.9.3")
     end
 
     it "handles jruby" do
@@ -104,7 +128,7 @@ G
 
       bundle "platform --ruby"
 
-      out.should eq("ruby 1.8.7 (jruby 1.6.5)")
+      expect(out).to eq("ruby 1.8.7 (jruby 1.6.5)")
     end
 
     it "handles rbx" do
@@ -117,7 +141,7 @@ G
 
       bundle "platform --ruby"
 
-      out.should eq("ruby 1.8.7 (rbx 1.2.4)")
+      expect(out).to eq("ruby 1.8.7 (rbx 1.2.4)")
     end
 
     it "raises an error if engine is used but engine version is not" do
@@ -128,9 +152,9 @@ G
         gem "foo"
       G
 
-      bundle "platform", :exitstatus => true
+      bundle "platform"
 
-      exitstatus.should_not == 0
+      expect(exitstatus).not_to eq(0) if exitstatus
     end
 
     it "raises an error if engine_version is used but engine is not" do
@@ -141,12 +165,12 @@ G
         gem "foo"
       G
 
-      bundle "platform", :exitstatus => true
+      bundle "platform"
 
-      exitstatus.should_not == 0
+      expect(exitstatus).not_to eq(0) if exitstatus
     end
 
-    it "raises an error if engine version doesn't match ruby version for mri" do
+    it "raises an error if engine version doesn't match ruby version for MRI" do
       gemfile <<-G
         source "file://#{gem_repo1}"
         ruby "1.8.7", :engine => 'ruby', :engine_version => '1.2.4'
@@ -154,9 +178,9 @@ G
         gem "foo"
       G
 
-      bundle "platform", :exitstatus => true
+      bundle "platform"
 
-      exitstatus.should_not == 0
+      expect(exitstatus).not_to eq(0) if exitstatus
     end
 
     it "should print if no ruby version is specified" do
@@ -167,31 +191,86 @@ G
       G
 
       bundle "platform --ruby"
-      puts err
 
-      out.should eq("No ruby version specified")
+      expect(out).to eq("No ruby version specified")
+    end
+
+    it "handles when there is a locked requirement" do
+      gemfile <<-G
+        ruby "< 1.8.7"
+      G
+
+      lockfile <<-L
+        GEM
+          specs:
+
+        PLATFORMS
+          ruby
+
+        DEPENDENCIES
+
+        RUBY VERSION
+           ruby 1.0.0p127
+
+        BUNDLED WITH
+           #{Bundler::VERSION}
+      L
+
+      bundle! "platform --ruby"
+      expect(out).to eq("ruby 1.0.0p127")
+    end
+
+    it "handles when there is a requirement in the gemfile" do
+      gemfile <<-G
+        ruby ">= 1.8.7"
+      G
+
+      bundle! "platform --ruby"
+      expect(out).to eq("ruby 1.8.7")
+    end
+
+    it "handles when there are multiple requirements in the gemfile" do
+      gemfile <<-G
+        ruby ">= 1.8.7", "< 2.0.0"
+      G
+
+      bundle! "platform --ruby"
+      expect(out).to eq("ruby 1.8.7")
     end
   end
 
   let(:ruby_version_correct) { "ruby \"#{RUBY_VERSION}\", :engine => \"#{local_ruby_engine}\", :engine_version => \"#{local_engine_version}\"" }
   let(:ruby_version_correct_engineless) { "ruby \"#{RUBY_VERSION}\"" }
+  let(:ruby_version_correct_patchlevel) { "#{ruby_version_correct}, :patchlevel => '#{RUBY_PATCHLEVEL}'" }
   let(:ruby_version_incorrect) { "ruby \"#{not_local_ruby_version}\", :engine => \"#{local_ruby_engine}\", :engine_version => \"#{not_local_ruby_version}\"" }
   let(:engine_incorrect) { "ruby \"#{RUBY_VERSION}\", :engine => \"#{not_local_tag}\", :engine_version => \"#{RUBY_VERSION}\"" }
   let(:engine_version_incorrect) { "ruby \"#{RUBY_VERSION}\", :engine => \"#{local_ruby_engine}\", :engine_version => \"#{not_local_engine_version}\"" }
+  let(:patchlevel_incorrect) { "#{ruby_version_correct}, :patchlevel => '#{not_local_patchlevel}'" }
+  let(:patchlevel_fixnum) { "#{ruby_version_correct}, :patchlevel => #{RUBY_PATCHLEVEL}1" }
 
-  def should_be_ruby_version_incorrect(opts = {:exitstatus => true})
-    exitstatus.should eq(18) if opts[:exitstatus]
-    out.should be_include("Your Ruby version is #{RUBY_VERSION}, but your Gemfile specified #{not_local_ruby_version}")
+  def should_be_ruby_version_incorrect
+    expect(exitstatus).to eq(18) if exitstatus
+    expect(out).to be_include("Your Ruby version is #{RUBY_VERSION}, but your Gemfile specified #{not_local_ruby_version}")
   end
 
-  def should_be_engine_incorrect(opts = {:exitstatus => true})
-    exitstatus.should eq(18) if opts[:exitstatus]
-    out.should be_include("Your Ruby engine is #{local_ruby_engine}, but your Gemfile specified #{not_local_tag}")
+  def should_be_engine_incorrect
+    expect(exitstatus).to eq(18) if exitstatus
+    expect(out).to be_include("Your Ruby engine is #{local_ruby_engine}, but your Gemfile specified #{not_local_tag}")
   end
 
-  def should_be_engine_version_incorrect(opts = {:exitstatus => true})
-    exitstatus.should eq(18) if opts[:exitstatus]
-    out.should be_include("Your #{local_ruby_engine} version is #{local_engine_version}, but your Gemfile specified #{local_ruby_engine} #{not_local_engine_version}")
+  def should_be_engine_version_incorrect
+    expect(exitstatus).to eq(18) if exitstatus
+    expect(out).to be_include("Your #{local_ruby_engine} version is #{local_engine_version}, but your Gemfile specified #{local_ruby_engine} #{not_local_engine_version}")
+  end
+
+  def should_be_patchlevel_incorrect
+    expect(exitstatus).to eq(18) if exitstatus
+    expect(out).to be_include("Your Ruby patchlevel is #{RUBY_PATCHLEVEL}, but your Gemfile specified #{not_local_patchlevel}")
+  end
+
+  def should_be_patchlevel_fixnum
+    expect(exitstatus).to eq(18) if exitstatus
+    expect(out).to be_include("The Ruby patchlevel in your Gemfile must be a string")
   end
 
   context "bundle install" do
@@ -203,7 +282,7 @@ G
         #{ruby_version_correct}
       G
 
-      bundled_app('Gemfile.lock').should exist
+      expect(bundled_app("Gemfile.lock")).to exist
     end
 
     it "installs fine with any engine" do
@@ -215,47 +294,70 @@ G
           #{ruby_version_correct_engineless}
         G
 
-        bundled_app('Gemfile.lock').should exist
+        expect(bundled_app("Gemfile.lock")).to exist
       end
     end
 
+    it "installs fine when the patchlevel matches" do
+      install_gemfile <<-G
+        source "file://#{gem_repo1}"
+        gem "rack"
+
+        #{ruby_version_correct_patchlevel}
+      G
+
+      expect(bundled_app("Gemfile.lock")).to exist
+    end
+
     it "doesn't install when the ruby version doesn't match" do
-      install_gemfile <<-G, :exitstatus => true
+      install_gemfile <<-G
         source "file://#{gem_repo1}"
         gem "rack"
 
         #{ruby_version_incorrect}
       G
 
-      bundled_app('Gemfile.lock').should_not exist
+      expect(bundled_app("Gemfile.lock")).not_to exist
       should_be_ruby_version_incorrect
     end
 
     it "doesn't install when engine doesn't match" do
-      install_gemfile <<-G, :exitstatus => true
+      install_gemfile <<-G
         source "file://#{gem_repo1}"
         gem "rack"
 
         #{engine_incorrect}
       G
 
-      bundled_app('Gemfile.lock').should_not exist
+      expect(bundled_app("Gemfile.lock")).not_to exist
       should_be_engine_incorrect
     end
 
     it "doesn't install when engine version doesn't match" do
       simulate_ruby_engine "jruby" do
-        install_gemfile <<-G, :exitstatus => true
+        install_gemfile <<-G
           source "file://#{gem_repo1}"
           gem "rack"
 
           #{engine_version_incorrect}
         G
 
-        bundled_app('Gemfile.lock').should_not exist
+        expect(bundled_app("Gemfile.lock")).not_to exist
         should_be_engine_version_incorrect
       end
     end
+
+    it "doesn't install when patchlevel doesn't match" do
+      install_gemfile <<-G
+        source "file://#{gem_repo1}"
+        gem "rack"
+
+        #{patchlevel_incorrect}
+      G
+
+      expect(bundled_app("Gemfile.lock")).not_to exist
+      should_be_patchlevel_incorrect
+    end
   end
 
   context "bundle check" do
@@ -272,9 +374,9 @@ G
         #{ruby_version_correct}
       G
 
-      bundle :check, :exitstatus => true
-      exitstatus.should eq(0)
-      out.should == "The Gemfile's dependencies are satisfied"
+      bundle :check
+      expect(exitstatus).to eq(0) if exitstatus
+      expect(out).to eq("Resolving dependencies...\nThe Gemfile's dependencies are satisfied")
     end
 
     it "checks fine with any engine" do
@@ -291,9 +393,9 @@ G
           #{ruby_version_correct_engineless}
         G
 
-        bundle :check, :exitstatus => true
-        exitstatus.should eq(0)
-        out.should == "The Gemfile's dependencies are satisfied"
+        bundle :check
+        expect(exitstatus).to eq(0) if exitstatus
+        expect(out).to eq("Resolving dependencies...\nThe Gemfile's dependencies are satisfied")
       end
     end
 
@@ -310,7 +412,7 @@ G
         #{ruby_version_incorrect}
       G
 
-      bundle :check, :exitstatus => true
+      bundle :check
       should_be_ruby_version_incorrect
     end
 
@@ -327,7 +429,7 @@ G
         #{engine_incorrect}
       G
 
-      bundle :check, :exitstatus => true
+      bundle :check
       should_be_engine_incorrect
     end
 
@@ -345,10 +447,27 @@ G
           #{engine_version_incorrect}
         G
 
-        bundle :check, :exitstatus => true
+        bundle :check
         should_be_engine_version_incorrect
       end
     end
+
+    it "fails when patchlevel doesn't match" do
+      install_gemfile <<-G
+        source "file://#{gem_repo1}"
+        gem "rack"
+      G
+
+      gemfile <<-G
+        source "file://#{gem_repo1}"
+        gem "rack"
+
+        #{patchlevel_incorrect}
+      G
+
+      bundle :check
+      should_be_patchlevel_incorrect
+    end
   end
 
   context "bundle update" do
@@ -375,7 +494,7 @@ G
       end
 
       bundle "update"
-      should_be_installed "rack 1.2", "rack-obama 1.0", "activesupport 3.0"
+      expect(the_bundle).to include_gems "rack 1.2", "rack-obama 1.0", "activesupport 3.0"
     end
 
     it "updates fine with any engine" do
@@ -392,7 +511,7 @@ G
         end
 
         bundle "update"
-        should_be_installed "rack 1.2", "rack-obama 1.0", "activesupport 3.0"
+        expect(the_bundle).to include_gems "rack 1.2", "rack-obama 1.0", "activesupport 3.0"
       end
     end
 
@@ -408,7 +527,7 @@ G
         build_gem "activesupport", "3.0"
       end
 
-      bundle :update, :exitstatus => true
+      bundle :update
       should_be_ruby_version_incorrect
     end
 
@@ -424,7 +543,7 @@ G
         build_gem "activesupport", "3.0"
       end
 
-      bundle :update, :exitstatus => true
+      bundle :update
       should_be_engine_incorrect
     end
 
@@ -441,10 +560,25 @@ G
           build_gem "activesupport", "3.0"
         end
 
-        bundle :update, :exitstatus => true
+        bundle :update
         should_be_engine_version_incorrect
       end
     end
+
+    it "fails when patchlevel doesn't match" do
+      gemfile <<-G
+        source "file://#{gem_repo1}"
+        gem "rack"
+
+        #{patchlevel_incorrect}
+      G
+      update_repo2 do
+        build_gem "activesupport", "3.0"
+      end
+
+      bundle :update
+      should_be_patchlevel_incorrect
+    end
   end
 
   context "bundle show" do
@@ -464,7 +598,7 @@ G
       G
 
       bundle "show rails"
-      out.should == default_bundle_path('gems', 'rails-2.3.2').to_s
+      expect(out).to eq(default_bundle_path("gems", "rails-2.3.2").to_s)
     end
 
     it "prints path if ruby version is correct for any engine" do
@@ -477,7 +611,7 @@ G
         G
 
         bundle "show rails"
-        out.should == default_bundle_path('gems', 'rails-2.3.2').to_s
+        expect(out).to eq(default_bundle_path("gems", "rails-2.3.2").to_s)
       end
     end
 
@@ -489,7 +623,7 @@ G
         #{ruby_version_incorrect}
       G
 
-      bundle "show rails", :exitstatus => true
+      bundle "show rails"
       should_be_ruby_version_incorrect
     end
 
@@ -501,7 +635,7 @@ G
         #{engine_incorrect}
       G
 
-      bundle "show rails", :exitstatus => true
+      bundle "show rails"
       should_be_engine_incorrect
     end
 
@@ -514,10 +648,25 @@ G
           #{engine_version_incorrect}
         G
 
-        bundle "show rails", :exitstatus => true
+        bundle "show rails"
         should_be_engine_version_incorrect
       end
     end
+
+    it "fails when patchlevel doesn't match" do
+      gemfile <<-G
+        source "file://#{gem_repo1}"
+        gem "rack"
+
+        #{patchlevel_incorrect}
+      G
+      update_repo2 do
+        build_gem "activesupport", "3.0"
+      end
+
+      bundle "show rails"
+      should_be_patchlevel_incorrect
+    end
   end
 
   context "bundle cache" do
@@ -537,7 +686,7 @@ G
       G
 
       bundle :cache
-      bundled_app("vendor/cache/rack-1.0.0.gem").should exist
+      expect(bundled_app("vendor/cache/rack-1.0.0.gem")).to exist
     end
 
     it "copies the .gem file to vendor/cache when ruby version matches for any engine" do
@@ -549,7 +698,7 @@ G
         G
 
         bundle :cache
-        bundled_app("vendor/cache/rack-1.0.0.gem").should exist
+        expect(bundled_app("vendor/cache/rack-1.0.0.gem")).to exist
       end
     end
 
@@ -560,7 +709,7 @@ G
         #{ruby_version_incorrect}
       G
 
-      bundle :cache, :exitstatus => true
+      bundle :cache
       should_be_ruby_version_incorrect
     end
 
@@ -571,22 +720,34 @@ G
         #{engine_incorrect}
       G
 
-      bundle :cache, :exitstatus => true
+      bundle :cache
       should_be_engine_incorrect
     end
 
     it "fails if the engine version doesn't match" do
       simulate_ruby_engine "jruby" do
         gemfile <<-G
-        gem 'rack'
+          gem 'rack'
 
-        #{engine_version_incorrect}
+          #{engine_version_incorrect}
         G
 
-        bundle :cache, :exitstatus => true
+        bundle :cache
         should_be_engine_version_incorrect
       end
     end
+
+    it "fails when patchlevel doesn't match" do
+      gemfile <<-G
+        source "file://#{gem_repo1}"
+        gem "rack"
+
+        #{patchlevel_incorrect}
+      G
+
+      bundle :cache
+      should_be_patchlevel_incorrect
+    end
   end
 
   context "bundle pack" do
@@ -606,7 +767,7 @@ G
       G
 
       bundle :pack
-      bundled_app("vendor/cache/rack-1.0.0.gem").should exist
+      expect(bundled_app("vendor/cache/rack-1.0.0.gem")).to exist
     end
 
     it "copies the .gem file to vendor/cache when ruby version matches any engine" do
@@ -618,7 +779,7 @@ G
         G
 
         bundle :pack
-        bundled_app("vendor/cache/rack-1.0.0.gem").should exist
+        expect(bundled_app("vendor/cache/rack-1.0.0.gem")).to exist
       end
     end
 
@@ -629,7 +790,7 @@ G
         #{ruby_version_incorrect}
       G
 
-      bundle :pack, :exitstatus => true
+      bundle :pack
       should_be_ruby_version_incorrect
     end
 
@@ -640,26 +801,39 @@ G
         #{engine_incorrect}
       G
 
-      bundle :pack, :exitstatus => true
+      bundle :pack
       should_be_engine_incorrect
     end
 
     it "fails if the engine version doesn't match" do
       simulate_ruby_engine "jruby" do
         gemfile <<-G
-        gem 'rack'
+          gem 'rack'
 
-        #{engine_version_incorrect}
+          #{engine_version_incorrect}
         G
 
-        bundle :pack, :exitstatus => true
+        bundle :pack
         should_be_engine_version_incorrect
       end
     end
+
+    it "fails when patchlevel doesn't match" do
+      gemfile <<-G
+        source "file://#{gem_repo1}"
+        gem "rack"
+
+        #{patchlevel_incorrect}
+      G
+
+      bundle :pack
+      should_be_patchlevel_incorrect
+    end
   end
 
   context "bundle exec" do
     before do
+      ENV["BUNDLER_FORCE_TTY"] = "true"
       system_gems "rack-1.0.0", "rack-0.9.1"
     end
 
@@ -671,7 +845,7 @@ G
       G
 
       bundle "exec rackup"
-      out.should == "0.9.1"
+      expect(out).to eq("0.9.1")
     end
 
     it "activates the correct gem when ruby version matches any engine" do
@@ -683,7 +857,7 @@ G
         G
 
         bundle "exec rackup"
-        out.should == "0.9.1"
+        expect(out).to eq("0.9.1")
       end
     end
 
@@ -694,7 +868,7 @@ G
         #{ruby_version_incorrect}
       G
 
-      bundle "exec rackup", :exitstatus => true
+      bundle "exec rackup"
       should_be_ruby_version_incorrect
     end
 
@@ -705,21 +879,33 @@ G
         #{engine_incorrect}
       G
 
-      bundle "exec rackup", :exitstatus => true
+      bundle "exec rackup"
       should_be_engine_incorrect
     end
 
-    it "fails when the engine version doesn't match" do
-      simulate_ruby_engine "jruby" do
-        gemfile <<-G
-          gem "rack", "0.9.1"
+    # it "fails when the engine version doesn't match" do
+    #   simulate_ruby_engine "jruby" do
+    #     gemfile <<-G
+    #       gem "rack", "0.9.1"
+    #
+    #       #{engine_version_incorrect}
+    #     G
+    #
+    #     bundle "exec rackup"
+    #     should_be_engine_version_incorrect
+    #   end
+    # end
 
-          #{engine_version_incorrect}
-        G
+    it "fails when patchlevel doesn't match" do
+      gemfile <<-G
+        source "file://#{gem_repo1}"
+        gem "rack"
 
-        bundle "exec rackup", :exitstatus => true
-        should_be_engine_version_incorrect
-      end
+        #{patchlevel_incorrect}
+      G
+
+      bundle "exec rackup"
+      should_be_patchlevel_incorrect
     end
   end
 
@@ -743,11 +929,11 @@ G
         #{ruby_version_correct}
       G
 
-      bundle "console" do |input|
+      bundle "console" do |input, _, _|
         input.puts("puts RACK")
         input.puts("exit")
       end
-      out.should include("0.9.1")
+      expect(out).to include("0.9.1")
     end
 
     it "starts IRB with the default group loaded when ruby version matches any engine" do
@@ -761,11 +947,11 @@ G
           #{ruby_version_correct_engineless}
         G
 
-        bundle "console" do |input|
+        bundle "console" do |input, _, _|
           input.puts("puts RACK")
           input.puts("exit")
         end
-        out.should include("0.9.1")
+        expect(out).to include("0.9.1")
       end
     end
 
@@ -779,7 +965,7 @@ G
         #{ruby_version_incorrect}
       G
 
-      bundle "console", :exitstatus => true
+      bundle "console"
       should_be_ruby_version_incorrect
     end
 
@@ -793,7 +979,7 @@ G
         #{engine_incorrect}
       G
 
-      bundle "console", :exitstatus => true
+      bundle "console"
       should_be_engine_incorrect
     end
 
@@ -808,10 +994,24 @@ G
           #{engine_version_incorrect}
         G
 
-        bundle "console", :exitstatus => true
+        bundle "console"
         should_be_engine_version_incorrect
       end
     end
+
+    it "fails when patchlevel doesn't match" do
+      gemfile <<-G
+        source "file://#{gem_repo1}"
+        gem "rack"
+        gem "activesupport", :group => :test
+        gem "rack_middleware", :group => :development
+
+        #{patchlevel_incorrect}
+      G
+
+      bundle "console"
+      should_be_patchlevel_incorrect
+    end
   end
 
   context "Bundler.setup" do
@@ -821,6 +1021,8 @@ G
         gem "yard"
         gem "rack", :group => :test
       G
+
+      ENV["BUNDLER_FORCE_TTY"] = "true"
     end
 
     it "makes a Gemfile.lock if setup succeeds" do
@@ -832,12 +1034,10 @@ G
         #{ruby_version_correct}
       G
 
-      File.read(bundled_app("Gemfile.lock"))
-
       FileUtils.rm(bundled_app("Gemfile.lock"))
 
       run "1"
-      bundled_app("Gemfile.lock").should exist
+      expect(bundled_app("Gemfile.lock")).to exist
     end
 
     it "makes a Gemfile.lock if setup succeeds for any engine" do
@@ -850,12 +1050,10 @@ G
           #{ruby_version_correct_engineless}
         G
 
-        File.read(bundled_app("Gemfile.lock"))
-
         FileUtils.rm(bundled_app("Gemfile.lock"))
 
         run "1"
-        bundled_app("Gemfile.lock").should exist
+        expect(bundled_app("Gemfile.lock")).to exist
       end
     end
 
@@ -868,23 +1066,15 @@ G
         #{ruby_version_incorrect}
       G
 
-      File.read(bundled_app("Gemfile.lock"))
-
       FileUtils.rm(bundled_app("Gemfile.lock"))
 
       ruby <<-R
         require 'rubygems'
-        require 'bundler'
-
-        begin
-          Bundler.setup
-        rescue Bundler::RubyVersionMismatch => e
-          puts e.message
-        end
+        require 'bundler/setup'
       R
 
-      bundled_app("Gemfile.lock").should_not exist
-      should_be_ruby_version_incorrect(:exitstatus => false)
+      expect(bundled_app("Gemfile.lock")).not_to exist
+      should_be_ruby_version_incorrect
     end
 
     it "fails when engine doesn't match" do
@@ -896,23 +1086,15 @@ G
         #{engine_incorrect}
       G
 
-      File.read(bundled_app("Gemfile.lock"))
-
       FileUtils.rm(bundled_app("Gemfile.lock"))
 
       ruby <<-R
         require 'rubygems'
-        require 'bundler'
-
-        begin
-          Bundler.setup
-        rescue Bundler::RubyVersionMismatch => e
-          puts e.message
-        end
+        require 'bundler/setup'
       R
 
-      bundled_app("Gemfile.lock").should_not exist
-      should_be_engine_incorrect(:exitstatus => false)
+      expect(bundled_app("Gemfile.lock")).not_to exist
+      should_be_engine_incorrect
     end
 
     it "fails when engine version doesn't match" do
@@ -925,25 +1107,37 @@ G
           #{engine_version_incorrect}
         G
 
-        File.read(bundled_app("Gemfile.lock"))
-
         FileUtils.rm(bundled_app("Gemfile.lock"))
 
         ruby <<-R
           require 'rubygems'
-          require 'bundler'
-
-          begin
-            Bundler.setup
-          rescue Bundler::RubyVersionMismatch => e
-            puts e.message
-          end
+          require 'bundler/setup'
         R
 
-        bundled_app("Gemfile.lock").should_not exist
-        should_be_engine_version_incorrect(:exitstatus => false)
+        expect(bundled_app("Gemfile.lock")).not_to exist
+        should_be_engine_version_incorrect
       end
     end
+
+    it "fails when patchlevel doesn't match" do
+      install_gemfile <<-G
+        source "file://#{gem_repo1}"
+        gem "yard"
+        gem "rack"
+
+        #{patchlevel_incorrect}
+      G
+
+      FileUtils.rm(bundled_app("Gemfile.lock"))
+
+      ruby <<-R
+        require 'rubygems'
+        require 'bundler/setup'
+      R
+
+      expect(bundled_app("Gemfile.lock")).not_to exist
+      should_be_patchlevel_incorrect
+    end
   end
 
   context "bundle outdated" do
@@ -955,7 +1149,7 @@ G
       install_gemfile <<-G
         source "file://#{gem_repo2}"
         gem "activesupport", "2.3.5"
-        gem "foo", :git => "#{lib_path('foo')}"
+        gem "foo", :git => "#{lib_path("foo")}"
       G
     end
 
@@ -968,14 +1162,14 @@ G
       gemfile <<-G
         source "file://#{gem_repo2}"
         gem "activesupport", "2.3.5"
-        gem "foo", :git => "#{lib_path('foo')}"
+        gem "foo", :git => "#{lib_path("foo")}"
 
         #{ruby_version_correct}
       G
 
       bundle "outdated"
-      out.should include("activesupport (3.0 > 2.3.5)")
-      out.should include("foo (1.0")
+      expect(out).to include("activesupport (newest 3.0, installed 2.3.5, requested = 2.3.5")
+      expect(out).to include("foo (newest 1.0")
     end
 
     it "returns list of outdated gems when the ruby version matches for any engine" do
@@ -988,14 +1182,14 @@ G
         gemfile <<-G
           source "file://#{gem_repo2}"
           gem "activesupport", "2.3.5"
-          gem "foo", :git => "#{lib_path('foo')}"
+          gem "foo", :git => "#{lib_path("foo")}"
 
           #{ruby_version_correct_engineless}
         G
 
         bundle "outdated"
-        out.should include("activesupport (3.0 > 2.3.5)")
-        out.should include("foo (1.0")
+        expect(out).to include("activesupport (newest 3.0, installed 2.3.5, requested = 2.3.5)")
+        expect(out).to include("foo (newest 1.0")
       end
     end
 
@@ -1008,12 +1202,12 @@ G
       gemfile <<-G
         source "file://#{gem_repo2}"
         gem "activesupport", "2.3.5"
-        gem "foo", :git => "#{lib_path('foo')}"
+        gem "foo", :git => "#{lib_path("foo")}"
 
         #{ruby_version_incorrect}
       G
 
-      bundle "outdated", :exitstatus => true
+      bundle "outdated"
       should_be_ruby_version_incorrect
     end
 
@@ -1026,12 +1220,12 @@ G
       gemfile <<-G
         source "file://#{gem_repo2}"
         gem "activesupport", "2.3.5"
-        gem "foo", :git => "#{lib_path('foo')}"
+        gem "foo", :git => "#{lib_path("foo")}"
 
         #{engine_incorrect}
       G
 
-      bundle "outdated", :exitstatus => true
+      bundle "outdated"
       should_be_engine_incorrect
     end
 
@@ -1045,14 +1239,54 @@ G
         gemfile <<-G
           source "file://#{gem_repo2}"
           gem "activesupport", "2.3.5"
-          gem "foo", :git => "#{lib_path('foo')}"
+          gem "foo", :git => "#{lib_path("foo")}"
 
           #{engine_version_incorrect}
         G
 
-        bundle "outdated", :exitstatus => true
+        bundle "outdated"
         should_be_engine_version_incorrect
       end
     end
+
+    it "fails when the patchlevel doesn't match" do
+      simulate_ruby_engine "jruby" do
+        update_repo2 do
+          build_gem "activesupport", "3.0"
+          update_git "foo", :path => lib_path("foo")
+        end
+
+        gemfile <<-G
+          source "file://#{gem_repo2}"
+          gem "activesupport", "2.3.5"
+          gem "foo", :git => "#{lib_path("foo")}"
+
+          #{patchlevel_incorrect}
+        G
+
+        bundle "outdated"
+        should_be_patchlevel_incorrect
+      end
+    end
+
+    it "fails when the patchlevel is a fixnum" do
+      simulate_ruby_engine "jruby" do
+        update_repo2 do
+          build_gem "activesupport", "3.0"
+          update_git "foo", :path => lib_path("foo")
+        end
+
+        gemfile <<-G
+          source "file://#{gem_repo2}"
+          gem "activesupport", "2.3.5"
+          gem "foo", :git => "#{lib_path("foo")}"
+
+          #{patchlevel_fixnum}
+        G
+
+        bundle "outdated"
+        should_be_patchlevel_fixnum
+      end
+    end
   end
 end
